@@ -17,12 +17,12 @@ from collections.abc import Iterator
 from collections.abc import Sequence
 from datetime import datetime
 from datetime import timedelta
-from datetime import timezone
 from pathlib import Path
 
 import sqlalchemy as sa
 from nbxmpp import JID
 from sqlalchemy import delete
+from sqlalchemy import Row
 from sqlalchemy import select
 from sqlalchemy import union_all
 from sqlalchemy import update
@@ -53,6 +53,7 @@ from gajim.common.storage.base import AlchemyStorage
 from gajim.common.storage.base import timeit
 from gajim.common.storage.base import VALUE_MISSING
 from gajim.common.storage.base import with_session
+from gajim.common.storage.base import with_session_yield_from
 from gajim.common.util.datetime import FIRST_UTC_DATETIME
 from gajim.common.util.text import get_random_string
 
@@ -496,18 +497,20 @@ class MessageArchiveStorage(AlchemyStorage):
 
     @with_session
     @timeit
-    def get_conversation_jids(self, session: Session, account: str) -> Sequence[JID]:
+    def get_conversation_jids(
+        self, session: Session, account: str
+    ) -> Sequence[Row[tuple[JID, int]]]:
         fk_account_pk = self._get_account_pk(session, account)
 
         subq = (
-            select(Message.fk_remote_pk)
+            select(Message.fk_remote_pk, Message.type)
             .distinct()
             .where(Message.fk_account_pk == fk_account_pk)
         ).subquery()
 
-        stmt = select(Remote.jid).join(subq)
+        stmt = select(Remote.jid, subq.c.type).join(subq)
         self._explain(session, stmt)
-        return session.scalars(stmt).all()
+        return session.execute(stmt).fetchall()
 
     @with_session
     @timeit
@@ -649,7 +652,7 @@ class MessageArchiveStorage(AlchemyStorage):
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
-        min_time = datetime.now(timezone.utc) - timedelta(
+        min_time = datetime.now(dt.UTC) - timedelta(
             seconds=MAX_MESSAGE_CORRECTION_DELAY
         )
 
@@ -706,8 +709,44 @@ class MessageArchiveStorage(AlchemyStorage):
         return app.storage.archive.get_message_with_id(
             account, jid, reply_id)
 
+
     @with_session
     @timeit
+    def get_message_stanza_ids_from_occupant(
+        self,
+        session: Session,
+        account: str,
+        jid: JID,
+        occupant_id: str,
+    ) -> Sequence[str] | None:
+        """Get stanza IDs for all messages which were sent by a participant
+         with occupant_id.
+         """
+        fk_account_pk = self._get_account_pk(session, account)
+        fk_remote_pk = self._get_jid_pk(session, jid)
+        stmt = select(Occupant.pk).where(
+            Occupant.id == occupant_id,
+            Occupant.fk_remote_pk == fk_remote_pk,
+            Occupant.fk_account_pk == fk_account_pk,
+        )
+        fk_occupant_pk = session.scalar(stmt)
+        if fk_occupant_pk is None:
+            self._log.warning(
+                'Unable to get messages, unknown occupant-id: %s', occupant_id
+            )
+            return None
+
+        stmt = select(Message.stanza_id).where(
+            Message.fk_remote_pk == fk_remote_pk,
+            Message.fk_account_pk == fk_account_pk,
+            Message.fk_occupant_pk == fk_occupant_pk,
+            Message.stanza_id.isnot(None)
+        )
+        result = cast(Sequence[str] | None, session.scalars(stmt).all())
+        return result
+
+    @with_session_yield_from
+    @timeit
     def search_archive(
         self,
         session: Session,
@@ -743,7 +782,7 @@ class MessageArchiveStorage(AlchemyStorage):
         # TODO: Does not search in corrections
 
         if before is None:
-            before = datetime.now(timezone.utc)
+            before = datetime.now(dt.UTC)
 
         if after is None:
             after = FIRST_UTC_DATETIME
@@ -770,6 +809,7 @@ class MessageArchiveStorage(AlchemyStorage):
             stmt.where(
                 Message.text.ilike(f'%{query}%'),
                 Message.timestamp.between(after, before),
+                ~Message.moderation.has(),
             )
             .order_by(sa.desc(Message.timestamp), sa.desc(Message.pk))
             .execution_options(yield_per=25)
@@ -811,8 +851,8 @@ class MessageArchiveStorage(AlchemyStorage):
         _, days = calendar.monthrange(year, month)
         local_end = datetime.combine(datetime(year, month, days), dt.time.max)
 
-        start = local_start.astimezone(timezone.utc)
-        end = local_end.astimezone(timezone.utc)
+        start = local_start.astimezone(dt.UTC)
+        end = local_end.astimezone(dt.UTC)
 
         stmt = (
             select(
@@ -894,8 +934,8 @@ class MessageArchiveStorage(AlchemyStorage):
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
-        start = datetime.combine(date, dt.time.min).astimezone(timezone.utc)
-        end = datetime.combine(date, dt.time.max).astimezone(timezone.utc)
+        start = datetime.combine(date, dt.time.min).astimezone(dt.UTC)
+        end = datetime.combine(date, dt.time.max).astimezone(dt.UTC)
 
         stmt = (
             select(Message.pk, Message.timestamp)
@@ -923,7 +963,7 @@ class MessageArchiveStorage(AlchemyStorage):
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
-        recent = datetime.now(timezone.utc) - timedelta(days=30)
+        recent = datetime.now(dt.UTC) - timedelta(days=30)
 
         stmt = (
             select(Message.resource)
@@ -932,6 +972,7 @@ class MessageArchiveStorage(AlchemyStorage):
                 Message.fk_remote_pk == fk_remote_pk,
                 Message.fk_account_pk == fk_account_pk,
                 Message.timestamp > recent,
+                Message.direction == ChatDirection.INCOMING,
                 Message.resource.isnot(None),
                 Message.correction_id.is_(None),
             )
@@ -1065,7 +1106,7 @@ class MessageArchiveStorage(AlchemyStorage):
 
             fk_account_pk = self._get_account_pk(session, account)
 
-            now = datetime.now(timezone.utc)
+            now = datetime.now(dt.UTC)
             threshold = now - timedelta(seconds=max_age)
 
             stmt = select(Message).where(
@@ -1077,7 +1118,7 @@ class MessageArchiveStorage(AlchemyStorage):
 
             log.info('Removed messages older then %s', threshold.isoformat())
 
-    @with_session
+    @with_session_yield_from
     @timeit
     def get_messages_for_export(
         self, session: Session, account: str, jid: JID
@@ -1092,7 +1133,7 @@ class MessageArchiveStorage(AlchemyStorage):
                 Message.fk_remote_pk == fk_remote_pk,
                 Message.correction_id.is_(None),
             )
-            .order_by(sa.desc(Message.timestamp), sa.desc(Message.pk))
+            .order_by(Message.timestamp, Message.pk)
             .execution_options(yield_per=25)
         )
 
